/*
 ***************************************************************************
 *
 *	Module Name : rex5052_gpib.h
 *
 ***************************************************************************
 */

#ifndef __REX5052_GPIB_H
#define __REX5052_GPIB_H

#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<errno.h>
#include	<fcntl.h>
#include	<sys/time.h>
#include	<sys/types.h>
#include	<pthread.h>

// IOCTL for Linux
#define IOCTL_GP_GET_RESOURCE			0
#define IOCTL_GP_READ_PORT_UCHAR		1
#define IOCTL_GP_READ_PORT_USHORT		2
#define IOCTL_GP_READ_PORT_ULONG		3
#define IOCTL_GP_WRITE_PORT_UCHAR		4
#define IOCTL_GP_WRITE_PORT_USHORT		5
#define IOCTL_GP_WRITE_PORT_ULONG		6
#define IOCTL_GP_START_INTERRUPT		7
#define IOCTL_GP_STOP_INTERRUPT			8

typedef struct _CARD_RESOURCE {
	unsigned int		IOBaseLow;		// 
	unsigned int		IOBaseHigh;	// 
	unsigned int		IOLength;		// length
	unsigned int		IrqLevel;		// interrupt level
} CARD_RESOURCE, *PCARD_RESOURCE;

//typedef struct  DELM_PARAMETER {
//	PSZ		mode;
//	ULONG	dlm;
//}DELMPARAM, *PDELMPARAM;

typedef struct T_IOPORT_UCHAR {
	unsigned int		Offset;
	unsigned char		Value;
} IOPORT_UCHAR, *PIOPORT_UCHAR;

typedef struct T_IOPORT_USHORT {
	unsigned int		Offset;
	unsigned short	Value;
} IOPORT_USHORT, *PIOPORT_USHORT;

typedef struct T_IOPORT_ULONG {
	unsigned int		Offset;
	unsigned int		Value;
} IOPORT_ULONG, *PIOPORT_ULONG;

typedef void (*PAPIFUNC)(void);

int gp_attach();
int gp_detach(int num);
int gp_cardinfo(unsigned int *pIOBase, unsigned int *pIrqNo);

int gp_GetDeviceID(int hDev, unsigned short UnitId, char* pDevId);
unsigned short OutPort(short Reg, short bOutVal);
unsigned short InPort(short Reg);
int gp_wtb(char * buf);
int gp_dataout(char* buf, int bufLen, unsigned char DMAflag);
int gp_datain(char* buf, int bufLen, unsigned char DMAflag);
int gp_datain_End(void);
int gp_init(short GpAdrs, short IOBase, short IrqNo);
int gp_cli(void);
int gp_ren(void);
int gp_tmout(int SecTime);
int gp_delay(int UsecTime);
int gp_setdelay(int DelayTime);
int gp_clr(char * adrs);
int gp_myadr(void);
int gp_srqCheck(void);

int gp_wrt(char* adrs, char* buf);
int gp_wrtd(char* buf, int bufLenOrg);
void gp_wait(int WaitSecTime);
int gp_trg(char* adrs);
int gp_wsrq(int WaitSecTime);
int gp_wsrqb(int WaitSecTime);

int gp_rds(char* adrs, unsigned char* status_byte);
int gp_rds1(char* adrs, unsigned char* status_byte);
int gp_red(char* adrs, char* buf, int bufLen);
int gp_redd(char* buf, int bufLen);
int gp_redah(char* adrs, char* buf, int bufLen);
int gp_redrst(void);

int gp_lcl(char* adrs);
int gp_llo(void);
int gp_count(void);
int gp_delm(char *mode, unsigned int dlm);
void gp_strtodbl(unsigned char *bPoint, double *val);
void gp_strtoflt(unsigned char *bPoint, float *val);
int gp_tfrout(char *adrs, int bufLen, char *buf);
int gp_tfroutd(int bufLen, char *buf);
int gp_tfrout_unt(char *adrs, int bufLen, char *buf);
int gp_tfrin(char *adrs, int bufLen, char *buf);
int gp_tfrinit(char *adrs);
int gp_tfrins(int bufLen, char *buf);
void gp_tfrend(void);
int gp_findlstn(char *adrs, int adrsLen);

int gp_srq(int SrqMode, PAPIFUNC pFunc);

#if 0 // If you connect multiple units to use.
int U2GpOpenUnit(unsigned short UnitId);
int U2GpCloseUnit(int hDriver);
int U2GpEnumUnit(unsigned short *pUnitId, unsigned short MaxUnit);
unsigned short U2OutPort(int hDriver, unsigned short Reg, unsigned short bOutVal);
unsigned short U2InPort(int hDriver, unsigned short Reg);
int U2Gpinit(int hDriver, unsigned short GpAdrs);
int U2Gpcli(int hDriver);
int U2Gpren(int hDriver);
int U2Gpclr(int hDriver, char *adrs);
int U2Gpmyadr(int hDriver);
int U2Gpwrt(int hDriver, char *adrs, char *buf);
int U2Gptrg(int hDriver, char *adrs);
int U2Gpwsrq(int hDriver, int WaitSecTime);
int U2Gpwsrqb(int hDriver, int WaitSecTime);
int U2Gprds(int hDriver, char *adrs, unsigned char *status_byte);
int U2Gprds1(int hDriver, char *adrs, unsigned char *status_byte);
int U2Gpred(int hDriver, char *adrs, char *buf, int bufLen);
int U2Gpredd(int hDriver, char *buf, int bufLen);
int U2Gpredah(int hDriver, char *adrs, char *buf, int bufLen);
int U2Gpredrst(int hDriver);
int U2Gpsrq(int hDriver, int SrqMode, PAPIFUNC pFunc);
int U2Gplcl(int hDriver, char *adrs);
int U2Gpllo(int hDriver);
int U2Gptmout(int hDriver, int SecTime);
int U2Gpsetdelay(int hDriver, int DelayTime);
int U2Gpcount(int hDriver);
int U2Gpdelm(int hDriver, char *mode, unsigned int dlm);
int U2Gptfrout(int hDriver, char *adrs, int bufLen, char *buf);
int U2Gptfrout_unt(int hDriver, char *adrs, int bufLen, char *buf);
int U2Gptfrin(int hDriver, char *adrs, int bufLen, char *buf);
int U2Gptfrinit(int hDriver, char *adrs);
int U2Gptfrins(int hDriver, int bufLen, char *buf);
void U2Gptfrend(int hDriver);
int U2Gpwtb(int hDriver, char *buf);

int U2GpsrqCheck(int hDriver);
int U2Gpwrtd(int hDriver, char *buf, int bufLen);
int U2Gptfroutd(int hDriver, int bufLen, char *buf);
int U2Gpfindlstn(int hDriver, char *adrs, int adrsLen);
#endif

#endif // __REX5052_GPIB_H

// End of File
