/*****************************************************************************/
/*                                                                           */
/*  FILE        :sample.c                                                    */
/*  DATE        :Mon, Aug 18, 2008                                           */
/*  DESCRIPTION :rex5052_gpib Linux sample program                           */
/*  history:                                                                 */
/*     v1.0.0:  2000.11.09  original issue                                   */
/*     v1.0.1:  2008.08.18  modified interupt routine                        */
/*                          add delimiter test functions                     */
/*                                                                           */
/*****************************************************************************/

#include "rex5052_gpib.h"
#include "def9914.h"

//#define _USE_U2

#define MAX_DEVICES				16
#define MAX_FILE_SIZE			65536

const char selected_filename[] = "/home/REX-5052/rex5052_gpib/test";
int fd=0;
int nNo=99;
short MyGpibAdrs = 0;	// GPIB address for PCMCIA-GPIB
char GpAdrsBuf[12];	// GPIB address for target-GPIB

int SaveDataFile(const void* buf, int bufLen);
int FileOpenSetMem(void* buf, int bufLen);

void Cmd_OnCmdGpInit();
void Cmd_OnCmdStart();
void Cmd_OnStartInterrupt();
void Cmd_OnStartTriger();
void Cmd_OnEventOccur();
void Cmd_OnStopInterrupt();
void Cmd_OnCancel();

void	SelectFunction(void);
void	CallFunction(void);

/*
 ********************************************************************************
 *	関数名 : SaveDataFile
 *	機　能 : 受信データをファイルに保存する
 *	引　数 : buf    - 受信データを格納するバッファ
 *           bufLen - 受信データサイズ
 *	戻り値 : ファイルへ書き込んだバイト数を返す
 ********************************************************************************
 */
int SaveDataFile(const void* buf, int bufLen)
{
	FILE *fp;	/* (1)ファイルポインタの宣言 */

	/* (2)ファイルのオープン */
	/*  ここで、ファイルポインタを取得する */
	if ((fp = fopen(selected_filename, "wb")) == NULL) {
		printf("file open error!!\n");
		return 0;
		//exit(1);	/* (3)エラーの場合は通常、異常終了する */
	}
	
	/* (4)ファイルの書き*/
	bufLen = fwrite(buf, sizeof(char), bufLen, fp);
	fclose(fp);	/* (5)ファイルのクローズ */
	printf("fwrite = %s, size = %d\n", selected_filename, bufLen);
	
	return bufLen;
}

/*
 ********************************************************************************
 *	関数名 : FileOpenSetMem
 *	機　能 : ファイルからデータを読み出す
 *	引　数 : buf    - ファイルデータを格納するバッファ
 *           bufLen - ファイルから読み出すバイト数
 *	戻り値 : ファイルから読み出したバイト数を返す
 ********************************************************************************
 */
int FileOpenSetMem(void* buf, int bufLen)
{
	FILE *fp;	/* (1)ファイルポインタの宣言 */

	/* (2)ファイルのオープン */
	/*  ここで、ファイルポインタを取得する */
	if ((fp = fopen(selected_filename, "rb")) == NULL) {
		printf("file open error!!\n");
		return 0;
		//exit(1);	/* (3)エラーの場合は通常、異常終了する */
	}
	
	/* (4)ファイルの読み*/
	bufLen = fread(buf, sizeof(char), bufLen, fp);
	fclose(fp);	/* (5)ファイルのクローズ */
	
	printf("fread = %s, size = %d\n", selected_filename, bufLen);
	return bufLen;
}

/*
 ********************************************************************************
 *	関数名 : Cmd_OnCmdGpInit
 *	機　能 : HP3478Aの初期化処理
 *	引　数 : なし
 *	戻り値 : なし
 ********************************************************************************
 */
void Cmd_OnCmdGpInit()
{
	int		GpStatus;
	char	szCommand[] = "H0KM01";

	// Init GPIB controller
#ifdef _USE_U2
	GpStatus = U2Gpinit( fd, MyGpibAdrs );
#else
	GpStatus = gp_init( MyGpibAdrs, 0, 0 );
#endif
	if( GpStatus != 0 )
	{
		printf("gp_init() error [%d]\n", GpStatus);
		return;
	}

	// enable IFC
#ifdef _USE_U2
	GpStatus = U2Gpcli(fd);
#else
	GpStatus = gp_cli();
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_cli() error [%d]\n", GpStatus);
		return;
	}

	// enable REN
#ifdef _USE_U2
	GpStatus = U2Gpren(fd);
#else
	GpStatus = gp_ren();
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_ren() error [%d]\n", GpStatus);
		return;
	}

	// set 3 second GPIB bus timeout
#ifdef _USE_U2
	GpStatus = U2Gptmout(fd, 3);
#else
	GpStatus = gp_tmout(3);
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_tmout() error [%d]\n", GpStatus);
		return;
	}

	// Send SDC command
	// get address of GPIBdevice for HP3478A
	memset(GpAdrsBuf, 0x00, sizeof(GpAdrsBuf));
	sprintf(GpAdrsBuf, "3");
#ifdef _USE_U2
	GpStatus = U2Gpclr(fd, GpAdrsBuf);
#else
	GpStatus = gp_clr( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_clr() error [%d]\n", GpStatus);
		return;
	}

	// Send LLO command
#ifdef _USE_U2
	GpStatus = U2Gpllo(fd);
#else
	GpStatus = gp_llo();
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_llo() error [%d]\n", GpStatus);
		return;
	}

	// Send HP3478A GPIB command
#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, szCommand);
#else
	GpStatus = gp_wrt( GpAdrsBuf, szCommand );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return;
	}
}

/*
 ********************************************************************************
 *	関数名 : Cmd_OnCmdStart
 *	機　能 : HP3478Aのポーリング処理を開始する
 *	引　数 : なし
 *	戻り値 : なし
 ********************************************************************************
 */
void Cmd_OnCmdStart()
{
	int 	GpStatus;
	char  	RcvData[256];
	unsigned char	StatusByte[16];
	int		i;

	// do triger command
#ifdef _USE_U2
	GpStatus = U2Gptrg(fd, GpAdrsBuf);
#else
	GpStatus = gp_trg( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_trg() error [%d]\n", GpStatus);
		return;
	}
	// do interrupt status poling for 10Sec
#ifdef _USE_U2
	GpStatus = U2Gpwsrq(fd, 10);
#else
	GpStatus = gp_wsrq( 10 );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_wsrq() error [%d]\n", GpStatus);
		return;
	}
	// do serial pole... get status
#ifdef _USE_U2
	GpStatus = U2Gprds(fd, GpAdrsBuf, StatusByte);
#else
	GpStatus = gp_rds( GpAdrsBuf, StatusByte );
#endif
	if( GpStatus != 0 )
	{
		printf("gp_rds() error [%d]\n", GpStatus);
		return;
	}
	// read data from GPIB bus
	memset( RcvData, 0x00, sizeof(RcvData) );
#ifdef _USE_U2
	GpStatus = U2Gpred(fd, GpAdrsBuf, RcvData, sizeof(RcvData));
#else
	GpStatus = gp_red( GpAdrsBuf, RcvData, sizeof(RcvData) );
#endif
	if( GpStatus != 0 )
	{
		printf("gp_red() error [%d]\n", GpStatus);
		return;
	}
	for(i=0;i<sizeof(RcvData);i++){
		if(i%16==0){
			printf("\n");
		}
		printf("%02x ", RcvData[i]);
	}
}

/*
 ********************************************************************************
 *	関数名 : Cmd_OnStartInterrupt
 *	機　能 : HP3478Aの割込み処理を開始する
 *	引　数 : なし
 *	戻り値 : なし
 ********************************************************************************
 */
void Cmd_OnStartInterrupt()
{
	int	GpStatus;
	
#ifdef _USE_U2
	GpStatus = U2Gpsrq(fd, 1, Cmd_OnEventOccur);
#else
	GpStatus = gp_srq( 1/*ENABLE_SRQ_INTERRUPT*/, Cmd_OnEventOccur );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_srq() error [%d]\n", GpStatus);
	}

	sleep(1);
}

/*
 ********************************************************************************
 *	関数名 : Cmd_OnStartTriger
 *	機　能 : HP3478Aへデバイストリガを発行する
 *	引　数 : なし
 *	戻り値 : なし
 ********************************************************************************
 */
void Cmd_OnStartTriger()
{
	int	GpStatus;

#ifdef _USE_U2
	GpStatus = U2Gptrg(fd, GpAdrsBuf);
#else
	GpStatus = gp_trg( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_trg() error [%d]\n", GpStatus);
	}

	sleep(1);
}

/*
 ********************************************************************************
 *	関数名 : Cmd_OnEventOccur
 *	機　能 : SRQ割込み発生時に呼び出される処理
 *	引　数 : なし
 *	戻り値 : なし
 ********************************************************************************
 */
void Cmd_OnEventOccur()
{
	int 	GpStatus;
	char	RcvData[256];
	unsigned char	StatusByte[16];
	int		i;
	double	dblVal = 0;
	float	fltVal = 0;

#ifdef _USE_U2
	GpStatus = U2GpsrqCheck(fd);
#else
	GpStatus = gp_srqCheck();
#endif
	printf("after trg... gp_srqCheck = [%d]\n", GpStatus);

#ifdef _USE_U2
	U2Gpsrq(fd, 0, NULL);
#else
	gp_srq( 0/*DISABLE_SRQ_INTERRUPT*/, NULL );
#endif

	// do serial pole... get status
#ifdef _USE_U2
	GpStatus = U2Gprds(fd, GpAdrsBuf, StatusByte);
#else
	GpStatus = gp_rds( GpAdrsBuf, StatusByte );
	//GpStatus = gp_rds1(GpAdrsBuf, StatusByte);
#endif
	if( GpStatus != 0 )
	{
		printf("gp_rds() error [%d]\n", GpStatus);
		return;
	}
	// read data from GPIB bus
	memset( RcvData, 0x00, sizeof(RcvData) );
#ifdef _USE_U2
	GpStatus = U2Gpred(fd, GpAdrsBuf, RcvData, sizeof(RcvData));
#else
	GpStatus = gp_red( GpAdrsBuf, RcvData, sizeof(RcvData) );
#endif
	if( GpStatus != 0 )
	{
		printf("gp_red() error [%d]\n", GpStatus);
		return;
	}
	
	for(i=0;i<sizeof(RcvData);i++){
		if(i%16==0){
			printf("\n");
		}
		printf("%02x ", RcvData[i]);
	}
	printf("\n");

	StatusByte[0] = 0x1B;
	StatusByte[1] = 0xDE;
	StatusByte[2] = 0x83;
	StatusByte[3] = 0x42;
	StatusByte[4] = 0xCA;
	StatusByte[5] = 0xC0;
	StatusByte[6] = 0xF3;
	StatusByte[7] = 0x3F;
	gp_strtodbl(StatusByte, &dblVal);
	printf("dblVal = %lf\n", dblVal);
	StatusByte[0] = 0x52;
	StatusByte[1] = 0x06;
	StatusByte[2] = 0x9E;
	StatusByte[3] = 0x3F;
	gp_strtoflt(StatusByte, &fltVal);
	printf("fltVal = %f\n", fltVal);
}

/*
 ********************************************************************************
 *	関数名 : Cmd_OnStopInterrupt
 *	機　能 : HP3478Aの割込み処理を中断する
 *	引　数 : なし
 *	戻り値 : なし
 ********************************************************************************
 */
void Cmd_OnStopInterrupt()
{
#ifdef _USE_U2
	U2Gpsrq(fd, 0, NULL);
#else
	gp_srq( 0/*DISABLE_SRQ_INTERRUPT*/, NULL );
#endif
}

/*
 ********************************************************************************
 *	関数名 : Cmd_OnCancel
 *	機　能 : HP3478Aの制御を終了する
 *	引　数 : なし
 *	戻り値 : なし
 ********************************************************************************
 */
void Cmd_OnCancel()
{
	int	GpStatus;

	//send LCL command
#ifdef _USE_U2
	//GpStatus = U2Gplcl(fd, "");
	GpStatus = U2Gplcl(fd, GpAdrsBuf);
#else
	GpStatus = gp_lcl( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_lcl() error [%d]\n", GpStatus);
	}
}

/*
 ********************************************************************************
 *	関数名 : Cmd_OnTest1
 *	機　能 : 複数の機器に対してSDCコマンドを発行するテスト
 *	引　数 : なし
 *	戻り値 : なし
 ********************************************************************************
 */
void Cmd_OnTest1()
{
	int	GpStatus;

	// Init GPIB controller
#ifdef _USE_U2
	GpStatus = U2Gpinit( fd, MyGpibAdrs );
#else
	GpStatus = gp_init( MyGpibAdrs, 0, 0 );
#endif
	if( GpStatus != 0 )
	{
		printf("gp_init() error [%d]\n", GpStatus);
		return;
	}

	// enable IFC
#ifdef _USE_U2
	GpStatus = U2Gpcli(fd);
#else
	GpStatus = gp_cli();
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_cli() error [%d]\n", GpStatus);
		return;
	}

	// enable REN
#ifdef _USE_U2
	GpStatus = U2Gpren(fd);
#else
	GpStatus = gp_ren();
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_ren() error [%d]\n", GpStatus);
		return;
	}

	// set 3 second GPIB bus timeout
#ifdef _USE_U2
	GpStatus = U2Gptmout(fd, 3);
#else
	GpStatus = gp_tmout(3);
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_tmout() error [%d]\n", GpStatus);
		return;
	}

	// Send SDC command to multi devices
	memset(GpAdrsBuf, 0x00, sizeof(GpAdrsBuf));
	sprintf(GpAdrsBuf, "3,5");
#ifdef _USE_U2
	GpStatus = U2Gpclr(fd, GpAdrsBuf);
#else
	GpStatus = gp_clr( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_clr() error [%d]\n", GpStatus);
		return;
	}
}

/*
 ********************************************************************************
 *	関数名 : Cmd_OnInitSlave
 *	機　能 : REX-5052をスレーブ機器として初期化する
 *	         (アドレスは4で固定, デリミタはCR+LF+EOI)
 *	引　数 : なし
 *	戻り値 : なし
 ********************************************************************************
 */
void Cmd_OnInitSlave()
{
	int	GpStatus;

	// set address of GPIBdevice for REX-5052
	MyGpibAdrs = 4;

	// Init GPIB controller
#ifdef _USE_U2
	GpStatus = U2Gpinit( fd, MyGpibAdrs );
#else
	GpStatus = gp_init( MyGpibAdrs, 0, 0 );
#endif
	if( GpStatus != 0 )
	{
		printf("gp_init() error [%d]\n", GpStatus);
		return;
	}
}

/*
 ********************************************************************************
 *	関数名 : Cmd_OnTxCmdSlave
 *	機　能 : REX-5052のスレーブ送信処理(*IDN?を送信)
 *	引　数 : なし
 *	戻り値 : なし
 ********************************************************************************
 */
void Cmd_OnTxCmdSlave()
{
	int	GpStatus;
	char TxBuf[MAX_FILE_SIZE];
	int TxLen;

	TxLen = 5;
	sprintf(TxBuf, "*IDN?");
	TxBuf[TxLen] = 0x00;

	printf("gp_wrtd() TxBuf %s\n", TxBuf);

#ifdef _USE_U2
	GpStatus = U2Gpwrtd(fd, TxBuf, TxLen);
#else
	GpStatus = gp_wrtd(TxBuf, TxLen);
#endif
	if( GpStatus != 0 )
	{
		printf("gp_wrtd() error [%d]\n", GpStatus);
		return;
	}
}

/*
 ********************************************************************************
 *	関数名 : Cmd_OnRxCmdSlave
 *	機　能 : REX-5052のスレーブ受信処理(受信後データを保存する)
 *	引　数 : なし
 *	戻り値 : なし
 ********************************************************************************
 */
void Cmd_OnRxCmdSlave()
{
	int	GpStatus;
	char RxBuf[MAX_FILE_SIZE];
	int RxLen = MAX_FILE_SIZE;

	memset(RxBuf, 0x00, MAX_FILE_SIZE);
#ifdef _USE_U2
	GpStatus = U2Gpredd( fd, RxBuf, RxLen );
#else
	GpStatus = gp_redd(RxBuf, RxLen);
#endif
	if( GpStatus != 0 )
	{
		printf("gp_redd() status [%d]\n", GpStatus);
	}

#ifdef _USE_U2
	GpStatus = U2Gpcount( fd );
#else
	GpStatus = gp_count();
#endif
	if( GpStatus != 0 )
	{
		printf("gp_count = [%d]\n", GpStatus);
		SaveDataFile(RxBuf, GpStatus);
	}
}

/*
 ********************************************************************************
 *	関数名 : Cmd_OnTxSlave
 *	機　能 : REX-5052のスレーブ送信処理(ファイルからデータを送信)
 *	引　数 : なし
 *	戻り値 : なし
 ********************************************************************************
 */
void Cmd_OnTxSlave()
{
	int	GpStatus;
	char TxBuf[MAX_FILE_SIZE];
	int TxLen;

	memset(TxBuf, 0x00, MAX_FILE_SIZE);
	TxLen = FileOpenSetMem(TxBuf, MAX_FILE_SIZE);
	if( TxLen == 0 )
	{
		return;
	}
	TxBuf[TxLen] = 0x00;

#ifdef _USE_U2
	GpStatus = U2Gptfroutd( fd, TxLen, TxBuf );
#else
	GpStatus = gp_tfroutd( TxLen, TxBuf );
#endif
	if( GpStatus != 0 )
	{
		printf("gp_tfroutd() error [%d]\n", GpStatus);
		return;
	}

}

/*
 ********************************************************************************
 *	関数名 : Cmd_OnRxSlave
 *	機　能 : REX-5052のスレーブ受信処理(受信後データを保存する)
 *	引　数 : なし
 *	戻り値 : なし
 ********************************************************************************
 */
void Cmd_OnRxSlave()
{
	int	GpStatus;
	char RxBuf[MAX_FILE_SIZE];
	int RxLen = MAX_FILE_SIZE;

	memset(RxBuf, 0x00, MAX_FILE_SIZE);
#ifdef _USE_U2
	GpStatus = U2Gptfrins( fd, RxLen, RxBuf );
#else
	GpStatus = gp_tfrins( RxLen, RxBuf );
#endif
	if( GpStatus != 0 )
	{
		printf("gp_tfrins() status [%d]\n", GpStatus);
	}

#ifdef _USE_U2
	GpStatus = U2Gpcount( fd );
#else
	GpStatus = gp_count();
#endif
	if( GpStatus != 0 )
	{
		printf("gp_count = [%d]\n", GpStatus);
		SaveDataFile(RxBuf, GpStatus);
	}
}

/*
 ********************************************************************************
 *	関数名 : Cmd_OnOutPort
 *	機　能 : REX-5052のレジスタへ値を書き込む
 *	引　数 : なし
 *	戻り値 : なし
 ********************************************************************************
 */
void Cmd_OnOutPort()
{
	int	GpStatus;
	unsigned int scanData;
	unsigned short Offset;
	unsigned short Data;
	unsigned char kakucho;

	printf("REX-5052 Register Offset(0,2,4,6,8,a,c,e) = 0x");
	scanf("%x",&scanData);
	Offset = scanData;
	if(Offset == 0x2){
		printf("IMR2:0x1E EOSR:0x9E BCR:0x1F ACCR:0x9F else:0 \n");
		printf("kakucho = 0x");
		scanf("%x",&scanData);
		kakucho = scanData;
	}else{
		kakucho = 0;
	}
	printf("Data = 0x");
	scanf("%x",&scanData);
	Data = scanData;

#ifdef _USE_U2
	if(kakucho)	U2OutPort(fd, AUXCR, kakucho);
	GpStatus = U2OutPort( fd, Offset, Data );
	if(kakucho)	U2OutPort(fd, AUXCR, clrpi);
#else
	if(kakucho)	OutPort(AUXCR, kakucho);
	GpStatus = OutPort( Offset, Data );
	if(kakucho)	OutPort(AUXCR, clrpi);
#endif
	if(GpStatus > 0xFF){
		printf("OutPort() error [%d]\n", GpStatus);
	}else{
		printf("Offset = [%02x], kakucho = [%02x], Data = [%02x]\n", Offset, kakucho, Data);
	}
}

/*
 ********************************************************************************
 *	関数名 : Cmd_OnInPort
 *	機　能 : REX-5052のレジスタから値を読み出す
 *	引　数 : なし
 *	戻り値 : なし
 ********************************************************************************
 */
void Cmd_OnInPort()
{
	int	GpStatus;
	unsigned int Offset;
	unsigned short Data;
	unsigned char kakucho;

	printf("REX-5052 Register Offset(0,2,4,6,8,a,c,e) = 0x");
	scanf("%x",&Offset);
	if(Offset == ISR2 || Offset == SPSR){
		kakucho = 0x1E;
	}else{
		kakucho = 0;
	}

#ifdef _USE_U2
	if(kakucho)	U2OutPort(fd, AUXCR, kakucho);
	Data = U2InPort( fd, Offset );
	if(kakucho)	U2OutPort(fd, AUXCR, clrpi);
#else
	if(kakucho)	OutPort(AUXCR, kakucho);
	Data = InPort( Offset );
	if(kakucho)	OutPort(AUXCR, clrpi);
#endif
	if(Data > 0xFF){
		printf("InPort() error [%d]\n", GpStatus);
	}else{
		printf("Offset = [%02x], kakucho = [%02x], Data = [%02x]\n", Offset, kakucho, Data);
	}
}


/*
 ********************************************************************************
 *	関数名 : Cmd_OnSetDelmt
 *	機　能 : トーカのデリミタを指定する
 *	引　数 : なし
 *	戻り値 : なし
 ********************************************************************************
 */
void Cmd_OnSetDelmt()
{
	int	value;
	int	stat = 0;

	printf("Input gp_delm() argument : dlm = 0x");
	scanf("%x",&value);

	// Init GPIB controller
#ifdef _USE_U2
	stat = U2Gpdelm( fd, "t", value );
#else
	stat = gp_delm( "t", value );
#endif
	if( stat != 0 )
	{
		printf("gp_delm() error [%d]\n", stat);
		return;
	}
}

/*
 ********************************************************************************
 *	関数名 : Cmd_OnSetDelml
 *	機　能 : リスナのデリミタを指定する
 *	引　数 : なし
 *	戻り値 : なし
 ********************************************************************************
 */
void Cmd_OnSetDelml()
{
	int	value;
	int	stat = 0;

	printf("Input gp_delm() argument : dlm = 0x");
	scanf("%x",&value);

	// Init GPIB controller
#ifdef _USE_U2
	stat = U2Gpdelm( fd, "l", value );
#else
	stat = gp_delm( "l", value );
#endif
	if( stat != 0 )
	{
		printf("gp_delm() error [%d]\n", stat);
		return;
	}
}

/*
 ********************************************************************************
 *	関数名 : Cmd_OnSetDelmb
 *	機　能 : トーカ・リスナ両方のデリミタを指定する
 *	引　数 : なし
 *	戻り値 : なし
 ********************************************************************************
 */
void Cmd_OnSetDelmb()
{
	int	value;
	int	stat = 0;

	printf("Input gp_delm() argument : dlm = 0x");
	scanf("%x",&value);

	// Init GPIB controller
#ifdef _USE_U2
	stat = U2Gpdelm( fd, "b", value );
#else
	stat = gp_delm( "b", value );
#endif
	if( stat != 0 )
	{
		printf("gp_delm() error [%d]\n", stat);
		return;
	}
}

/*
 ********************************************************************************
 *	関数名 : main
 *	機　能 : メインルーチン
 *	引　数 : なし
 *	戻り値 : -
 ********************************************************************************
 */
int main()
{
	while(1) {
		SelectFunction();
		CallFunction();
	}
	return 0;
}

/*
 ********************************************************************************
 *	関数名 : SelectFunction
 *	機　能 : 処理の選択を行う
 *	引　数 : なし
 *	戻り値 : なし
 ********************************************************************************
 */
void SelectFunction(void)
{
	printf("\n");
	printf(" 0 open \n");
	printf(" 1 close \n");
	printf(" 2 GET_RESOURCE \n");
	printf(" 3 READ_PORT_UCHAR \n");
	printf(" 4 READ_PORT_USHORT \n");
	printf(" 5 WRITE_PORT_UCHAR \n");
	printf(" 6 WRITE_PORT_USHORT \n");
	printf(" 7 HP3478A(poling/int) Init \n");
	printf(" 8 HP3478A(poling)     Start Poling \n");
	printf(" 9 HP3478A(int)        Start Interrupt \n");
	printf("10 HP3478A(int)        Start Triger \n");
	printf("11 HP3478A(int)        Stop Interrupt \n");
	printf("12 HP3478A(poling/int) Exit \n");
	/*
	// 以下の処理は使用できるが表示は行わない
	printf("\n");
	printf("14 Init REX-5052 as Slave \n");
	printf("15 gp_wrtd() as Slave\n");
	printf("16 gp_redd() as Slave\n");
	printf("17 gp_tfroutd() as Slave\n");
	printf("18 gp_tfrins() as Slave\n");

	printf("19 OutPort()\n");
	printf("20 InPort()\n");
	printf("21 gp_delm(""t"")\n");
	printf("22 gp_delm(""l"")\n");
	printf("23 gp_delm(""b"")\n");
	*/
	printf("99 exit \n");

	// SelectCard
  	while(1) {
		nNo=99;
		printf("No = ");
		scanf("%d",&nNo);
 		if(nNo == 99) exit(0);
		if(nNo >= 0) break;
	}
}

/*
 ********************************************************************************
 *	関数名 : CallFunction
 *	機　能 : SelectFunctionで選択した処理を実行する
 *	引　数 : なし
 *	戻り値 : なし
 ********************************************************************************
 */
void	CallFunction(void)
{
	int stat;
	int offset, value;
	unsigned int iobase, irq;
	IOPORT_UCHAR		bIOPort;
	IOPORT_USHORT		wIOPort;

	switch(nNo){
		case 0:
			printf(" 0	open;\n");
			if(fd>0){
                printf("Device already opened \n");
                break;
            }
#ifdef _USE_U2
			MaxUnit = U2GpEnumUnit(UnitId, MaxUnit);
			for(i=0;i<MaxUnit;i++){
				printf("U2GpEnumUnit%d UnitId = %02x \n", i, UnitId[i]);
			}
			printf("select UnitId\n");
			scanf("%d",&id);
			fd=U2GpOpenUnit(id);
#else
			fd=gp_attach();
#endif
			if(fd<0){
				perror("");
			}
			printf("fd=%d \n",fd);
			//gp_GetDeviceID(fd, 0, (char*)&MyGpibAdrs);
			break;
		case 1:
			printf(" 1	close;\n");
			if(fd<=0){
                printf("Device not opened \n");
                break;
            }
#ifdef _USE_U2
			stat=U2GpCloseUnit(fd);
#else
			stat=gp_detach(fd);
#endif
			if(stat<0){
				perror("close error");
			}
			else{
				fd=0;
            }
			break;
		case 2:
			printf(" 2	 get card resource \n");

			stat=gp_cardinfo(&iobase, &irq);
			if(stat<0){
				printf("get card resource error\n");
				break;
			}
			printf("GET_RESOURCE IOBase = 0x%x, Irq = %d\n", iobase, irq);
			break;
		case 3:
			printf(" 3	READ_PORT_UCHAR \n");
			printf("offset = ");
			scanf("%d",&offset);
			bIOPort.Offset = offset;

			stat=ioctl(fd, IOCTL_GP_READ_PORT_UCHAR, &bIOPort);
			if(stat<0){
				perror("");
			}
			printf("READ_PORT_UCHAR: offset = 0x%x value = 0x%x\n", 
				bIOPort.Offset, bIOPort.Value);
			break;
		case 4:
			printf(" 4 READ_PORT_USHORT \n");
			printf("offset = ");
			scanf("%d",&offset);
			wIOPort.Offset = offset;

			stat=ioctl(fd, IOCTL_GP_READ_PORT_USHORT, &wIOPort);
			if(stat<0){
				perror("");
			}
			printf("READ_PORT_USHORT: offset = 0x%x value = 0x%x\n", 
				wIOPort.Offset, wIOPort.Value);
			break;
		case 5:
			printf(" 5 WRITE_PORT_UCHAR \n");
			printf("offset = ");
			scanf("%d",&offset);
			bIOPort.Offset = offset;
			printf("value = ");
			scanf("%d",&value);
			bIOPort.Value = value;

			stat=ioctl(fd, IOCTL_GP_WRITE_PORT_UCHAR, &bIOPort);
			if(stat<0){
				perror("");
			}
			printf("WRITE_PORT_UCHAR: offset = 0x%x value = 0x%x\n", 
				bIOPort.Offset, bIOPort.Value);
			break;
		case 6:
			printf(" 6 WRITE_PORT_USHORT \n");
			printf("offset = ");
			scanf("%d",&offset);
			wIOPort.Offset = offset;
			printf("value = ");
			scanf("%d",&value);
			wIOPort.Value = value;

			stat=ioctl(fd, IOCTL_GP_WRITE_PORT_USHORT, &wIOPort);
			if(stat<0){
				perror("");
			}
			printf("WRITE_PORT_USHORT: offset = 0x%x value = 0x%x\n", 
				wIOPort.Offset, wIOPort.Value);
			break;
		case 7:
			printf(" 7 HP3478A Init \n");
			Cmd_OnCmdGpInit();
			break;
		case 8:
			printf(" 8 HP3478A Start Poling \n");
			Cmd_OnCmdStart();
			break;
		case 9:
			printf(" 9 HP3478A Start Interrupt \n");
			Cmd_OnStartInterrupt();
			break;
		case 10:
			printf("10 HP3478A Start Triger \n");
			Cmd_OnStartTriger();
			break;
		case 11:
			printf("11 HP3478A Stop Interrupt \n");
			Cmd_OnStopInterrupt();
			break;
		case 12:
			printf("12 HP3478A Exit \n");
			Cmd_OnCancel();
			break;
		case 13:
			printf("13 send cmd to multi devices \n");
			Cmd_OnTest1();
			break;
		case 14:
			printf("14 Init REX-5052 as Slave \n");
			Cmd_OnInitSlave();
			break;
		case 15:
			printf("15 gp_wrtd() as Slave\n");
			Cmd_OnTxCmdSlave();
			break;
		case 16:
			printf("16 gp_redd() as Slave\n");
			Cmd_OnRxCmdSlave();
			break;
		case 17:
			printf("17 gp_tfroutd() as Slave\n");
			Cmd_OnTxSlave();
			break;
		case 18:
			printf("18 gp_tfrins() as Slave\n");
			Cmd_OnRxSlave();
			break;
		case 19:
			printf("19 OutPort()\n");
			Cmd_OnOutPort();
			break;
		case 20:
			printf("20 InPort()\n");
			Cmd_OnInPort();
			break;
		case 21:
			printf("21 gp_delm(""t"")\n");
			Cmd_OnSetDelmt();
			break;
		case 22:
			printf("22 gp_delm(""l"")\n");
			Cmd_OnSetDelml();
			break;
		case 23:
			printf("23 gp_delm(""b"")\n");
			Cmd_OnSetDelmb();
			break;
		default:
			break;

	}
}



