//gcc -g hp3478a_int.c -o hp3478a_int -L/usr/local/lib/ -lrex5052_gpib -lpthread \
//	`pkg-config gtk+-2.0 --cflags --libs`
// caution: You need ROOT previledge when execute this utility
#include <stdlib.h>
#include <gtk/gtk.h>
#include <glib.h>

#include "rex5052_gpib.h"

//#define _USE_U2
#define MAX_DEVICES				16


int fd=0;
short MyGpibAdrs = 1;	// GPIB address for PCMCIA-GPIB
char GpAdrsBuf[12];	// GPIB address for target-GPIB
int InitFlag;
GtkWidget *status_bar;
gint context_id;



void Cmd_OnEventOccur()
{
	int 	GpStatus;
	char  	RcvData[256];
	unsigned char	StatusByte[16];
	//int	length;
	//char	*pbuf;
	//int	i;
	gchar *buff;

#ifdef _USE_U2
	U2Gpsrq(fd, 0, NULL);
#else
	gp_srq( 0/*DISABLE_SRQ_INTERRUPT*/, NULL );
#endif

	// do serial pole... get status
#ifdef _USE_U2
	GpStatus = U2Gprds(fd, GpAdrsBuf, StatusByte);
#else
	GpStatus = gp_rds( GpAdrsBuf, StatusByte );
#endif
	if( GpStatus != 0 )
	{
		printf("gp_rds() error [%d]\n", GpStatus);
		return;
	}
	// read data from GPIB bus
	memset( RcvData, 0x00, sizeof(RcvData) );
#ifdef _USE_U2
	GpStatus = U2Gpred(fd, GpAdrsBuf, RcvData, sizeof(RcvData));
#else
	GpStatus = gp_red( GpAdrsBuf, RcvData, sizeof(RcvData) );
#endif
	if( GpStatus != 0 )
	{
		printf("gp_red() error [%d]\n", GpStatus);
		return;
	}

#if 1
	RcvData[11] = 0x00;
	buff = g_strdup_printf ("%s", RcvData);
	gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, buff);
	g_free (buff);
#else
	length = gp_count();
	for(i=0;i<length;i++){
		if(i%16==0){
			printf("\n");
		}
		printf("%02x ", RcvData[i]);
	}
	printf("\nlength = %d\n", length);
	pbuf = (char *)malloc(length);
	memcpy(pbuf, RcvData, length);
	buff = g_strdup_printf ("%s", pbuf);
	gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, buff);
	g_free (buff);
	free(pbuf);
#endif
}

static void Cmd_OnCmdGpInit( GtkWidget *widget,
                       gpointer   data )
{
	int		GpStatus;
	char	szCommand[] = "H0KM01";

	// Init GPIB controller
#ifdef _USE_U2
	GpStatus = U2Gpinit( fd, MyGpibAdrs );
#else
	GpStatus = gp_init( MyGpibAdrs, 0, 0 );
#endif
	if( GpStatus != 0 )
	{
		printf("gp_init() error [%d]\n", GpStatus);
		return;
	}

	// enable IFC
#ifdef _USE_U2
	GpStatus = U2Gpcli(fd);
#else
	GpStatus = gp_cli();
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_cli() error [%d]\n", GpStatus);
		return;
	}

	// enable REN
#ifdef _USE_U2
	GpStatus = U2Gpren(fd);
#else
	GpStatus = gp_ren();
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_ren() error [%d]\n", GpStatus);
		return;
	}

	// set 3 second GPIB bus timeout
#ifdef _USE_U2
	GpStatus = U2Gptmout(fd, 3);
#else
	GpStatus = gp_tmout(3);
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_tmout() error [%d]\n", GpStatus);
		return;
	}

	// Send SDC command
	// get address of GPIBdevice for HP3478A
#ifdef _USE_U2
	GpStatus = U2Gpclr(fd, GpAdrsBuf);
#else
	GpStatus = gp_clr( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_clr() error [%d]\n", GpStatus);
		return;
	}

	// Send LLO command
#ifdef _USE_U2
	GpStatus = U2Gpllo(fd);
#else
	GpStatus = gp_llo();
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_llo() error [%d]\n", GpStatus);
		return;
	}

	// Send HP3478A GPIB command
#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, szCommand);
#else
	GpStatus = gp_wrt( GpAdrsBuf, szCommand );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return;
	}

	InitFlag = 1;
}

static void Cmd_OnStartInterrupt( GtkWidget *widget,
                      gpointer   data )
{
	int	GpStatus;

	if(InitFlag == 0){
		return;
	}
#ifdef _USE_U2
	GpStatus = U2Gpsrq(fd, 1, Cmd_OnEventOccur);
#else
	GpStatus = gp_srq( 1/*ENABLE_SRQ_INTERRUPT*/, Cmd_OnEventOccur );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_srq() error [%d]\n", GpStatus);
	}

	sleep(1);
}

static void Cmd_OnStartTriger( GtkWidget *widget,
                      gpointer   data )
{
	int	GpStatus;

	if(InitFlag == 0){
		return;
	}
#ifdef _USE_U2
	GpStatus = U2Gptrg(fd, GpAdrsBuf);
#else
	GpStatus = gp_trg( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_trg() error [%d]\n", GpStatus);
	}

	sleep(1);
}

static void Cmd_OnStopInterrupt( GtkWidget *widget,
                      gpointer   data )
{
	if(InitFlag == 0){
		return;
	}
#ifdef _USE_U2
	U2Gpsrq(fd, 0, NULL);
#else
	gp_srq( 0/*DISABLE_SRQ_INTERRUPT*/, NULL );
#endif
}

static void Cmd_OnCancel( GtkWidget *widget,
                      gpointer   data )
{
	int	GpStatus;

	if(InitFlag){
		//send LCL command
#ifdef _USE_U2
		GpStatus = U2Gplcl(fd, GpAdrsBuf);
#else
		GpStatus = gp_lcl( GpAdrsBuf );
#endif
		if ( GpStatus != 0 )
		{
			printf("gp_lcl() error [%d]\n", GpStatus);
		}
	}
}

static void enter_callback( GtkWidget *widget,
                            GtkWidget *entry )
{
  const gchar *entry_text;

  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  printf ("changed contents: %s\n", entry_text);
  memset(GpAdrsBuf, 0x00, sizeof(GpAdrsBuf));
  sprintf(GpAdrsBuf, "%s", entry_text);
}

static void create_window( void )
{
	GtkWidget *window;
	GtkWidget *hbox;
	GtkWidget *vbox;
	GtkWidget *label;
	GtkWidget *entry;
	GtkWidget *button;
	char	buf[12];
	gchar *buff;

    /* make new window */
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_container_set_border_width (GTK_CONTAINER (window), 5);
    gtk_widget_set_size_request (GTK_WIDGET (window), 465, 140);
    gtk_window_set_title (GTK_WINDOW (window), "hp3478a interrupt");
    g_signal_connect (G_OBJECT (window), "destroy",
		      G_CALLBACK (gtk_main_quit), NULL);

    vbox = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (window), vbox);
    gtk_widget_show (vbox);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 10);
    gtk_widget_show (hbox);

    label = gtk_label_new ("GPIB 機器アドレス");
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 5);
    gtk_widget_show (label);

    entry = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (entry), 1);
    g_signal_connect (G_OBJECT (entry), "changed",
		      G_CALLBACK (enter_callback),
		      (gpointer) entry);
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, "3");
    gtk_entry_set_text (GTK_ENTRY (entry), buf);
    gtk_box_pack_start (GTK_BOX (hbox), entry, FALSE, FALSE, 0);
    gtk_widget_show (entry);

    label = gtk_label_new ("REX-5052 アドレス");
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 10);
    gtk_widget_show (label);

    status_bar = gtk_statusbar_new ();
    context_id = gtk_statusbar_get_context_id(
                          GTK_STATUSBAR (status_bar), "Statusbar example");
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, " %d", MyGpibAdrs);
    gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, buf);
    gtk_box_pack_start (GTK_BOX (hbox), status_bar, TRUE, TRUE, 0);
    gtk_widget_show (status_bar);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 10);
    gtk_widget_show (hbox);

    label = gtk_label_new ("測定値");
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 10);
    gtk_widget_show (label);

    status_bar = gtk_statusbar_new ();
    context_id = gtk_statusbar_get_context_id(
                          GTK_STATUSBAR (status_bar), "Statusbar example");
    gtk_box_pack_start (GTK_BOX (hbox), status_bar, TRUE, TRUE, 10);
    gtk_widget_show (status_bar);

    label = gtk_label_new (" ");
    gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, FALSE, 0);
    gtk_widget_show (label);

    hbox = gtk_hbox_new (FALSE, 2);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 10);
    gtk_widget_show (hbox);

    button = gtk_button_new_with_label ("初期化");
    g_signal_connect (G_OBJECT (button), "clicked",
                      G_CALLBACK (Cmd_OnCmdGpInit), GINT_TO_POINTER (context_id));
    gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
    gtk_widget_show (button);              

    button = gtk_button_new_with_label ("割り込み開始");
    g_signal_connect (G_OBJECT (button), "clicked",
                      G_CALLBACK (Cmd_OnStartInterrupt), GINT_TO_POINTER (context_id));
    gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
    gtk_widget_show (button);

    button = gtk_button_new_with_label ("トリガー発行");
    g_signal_connect (G_OBJECT (button), "clicked",
                      G_CALLBACK (Cmd_OnStartTriger), GINT_TO_POINTER (context_id));
    gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
    gtk_widget_show (button);

    button = gtk_button_new_with_label ("割り込み中断");
    g_signal_connect (G_OBJECT (button), "clicked",
                      G_CALLBACK (Cmd_OnStopInterrupt), GINT_TO_POINTER (context_id));
    gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
    gtk_widget_show (button);

    button = gtk_button_new_with_label ("終了");
    g_signal_connect (G_OBJECT (button), "clicked",
		               G_CALLBACK (Cmd_OnCancel), GINT_TO_POINTER (context_id));
    g_signal_connect (G_OBJECT (button), "clicked",
			      G_CALLBACK (gtk_main_quit), NULL);
    gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
    gtk_widget_grab_default (button);
    gtk_widget_show (button);

    gtk_widget_show (window);
}

int main( int   argc,
          char *argv[] )
{
#ifdef _USE_U2
	int id, i;
	unsigned short UnitId[MAX_DEVICES];
	unsigned short MaxUnit = MAX_DEVICES;

	// 引数無しの場合
	if (argc != 2) {
		MaxUnit = U2GpEnumUnit(UnitId, MaxUnit);
		for(i=0;i<MaxUnit;i++){
			printf("U2GpEnumUnit%d UnitId = %02x \n", i, UnitId[i]);
		}
		//最初に見つけたREX-5052をオープン
		fd=U2GpOpenUnit(UnitId[0]);
	}else{
		id = atoi (argv[1]);
		//入力されたIDをもつREX-5052をオープン
		fd=U2GpOpenUnit(id);
	}
#else
	fd=gp_attach();
#endif
	if(fd<=0){
		printf("Device not opened \n");
		return 0;
    }
	InitFlag = 0;
	gp_GetDeviceID(fd, 0, (char*)&MyGpibAdrs);

	gtk_init (&argc, &argv);

	create_window();

	gtk_main ();
	gp_detach(fd);

	return 0;
}
