//gcc -g sc7201.c -o sc7201 -L/usr/local/lib/ -lrex5052_gpib -lpthread \
//	`pkg-config gtk+-2.0 --cflags --libs`
// caution: You need ROOT previledge when execute this utility
#include <stdlib.h>
#include <gtk/gtk.h>
#include <glib.h>

#include "rex5052_gpib.h"

//#define _USE_U2
#define MAX_DEVICES				16


int fd=0;
short MyGpibAdrs = 1;	// GPIB address for PCMCIA-GPIB
char GpAdrsBuf[12];	// GPIB address for target-GPIB
char szBuf[256];

int InitFlag;
int DiffFlag;
int RangeFlag;
int TimeFlag;

int timer = 0;
GtkWidget *counter_bar;
gint counter_id;


static int SC7201Command(void)
{
	int		GpStatus;

	memset( szBuf, 0x00, sizeof(szBuf) );
	switch( RangeFlag )
	{
		case 0:	strcpy( szBuf, "F1" );	break;	/* 周波数測定200MHz */
		case 1:	strcpy( szBuf, "F2" );	break;	/* 周波数測定16MHz */
		case 2:	strcpy( szBuf, "F3" );	break;	/* 周期測定 */
		case 3:	strcpy( szBuf, "F4" );	break;	/* 周波数比測定 */
		case 4:	strcpy( szBuf, "F6" );	break;	/* 時間間隔測定 */
		case 5:	strcpy( szBuf, "F5" );	break;	/* デューレシオ測定 */
		case 6:	strcpy( szBuf, "F7" );	break;	/* 加算計数測定 */
		case 7:	strcpy( szBuf, "F8" );	break;	/* 加算計数測定 OPEN */
		case 8:	strcpy( szBuf, "F9" );	break;	/* 加算計数測定 CLOSE */
	}
	switch( TimeFlag )
	{
		case 0:	strcat( szBuf, "R3" );	break;	/* 1ms */
		case 1:	strcat( szBuf, "R5" );	break;	/* 0.1s */
		case 2:	strcat( szBuf, "R6" );	break;	/* 1s */
		case 3:	strcat( szBuf, "R7" );	break;	/* 10s */
		case 4:	strcat( szBuf, "R0" );	break;	/* 外部ゲートB */
	}
	switch( DiffFlag )
	{
		case 0:	strcat( szBuf, "P0" );	break;	/* DIFF OFF */
		case 1:	strcat( szBuf, "P1" );	break;	/* DIFF ON */
	}

	// 作成したコマンドどおりにSC7201を設定する
	printf ("send command: %s\n", szBuf);
#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, szBuf);
#else
	GpStatus = gp_wrt( GpAdrsBuf, szBuf );
#endif
	return GpStatus;
}

static gboolean timeout_callback( gpointer data )
{
	int		GpStatus;
	int		length, i;

#ifdef _USE_U2
	GpStatus = U2Gptrg( fd, GpAdrsBuf );
#else
	GpStatus = gp_trg( GpAdrsBuf );
#endif
	if( GpStatus != 0 )
	{
		printf("gp_trg() error [%d]\n", GpStatus);
		return FALSE;
	}

	memset(szBuf,0x00,sizeof(szBuf));
#ifdef _USE_U2
	GpStatus = U2Gpred(fd, GpAdrsBuf, szBuf, sizeof(szBuf));
#else
	GpStatus = gp_red( GpAdrsBuf, szBuf, sizeof(szBuf) );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_red() error [%d]\n", GpStatus);
		return FALSE;
	}

#ifdef _USE_U2
	length = U2Gpcount(fd);
#else
	length = gp_count();
#endif
	for(i=0;i<length;i++){
		//if(i%16==0){
		//	printf("\n");
		//}
		//printf("%02x ", szBuf[i]);
		if(szBuf[i]==0x0d){
			szBuf[0] = 0x20;
			szBuf[1] = 0x20;
			szBuf[2] = 0x20;
			szBuf[3] = 0x20;
			szBuf[4] = 0x20;
			szBuf[i] = 0x00;
			break;
		}
	}
	gtk_statusbar_push (GTK_STATUSBAR (counter_bar), GPOINTER_TO_INT (counter_id), szBuf);

	/* これがタイムアウト関数なので、
	* この関数が連続して呼び出されるように TRUE を返します。*/
	return TRUE;
}

static void Cmd_OnCmdGpInit( GtkWidget *widget,
                       gpointer   data )
{
	int		GpStatus;

	// Init GPIB controller
#ifdef _USE_U2
	GpStatus = U2Gpinit( fd, MyGpibAdrs );
#else
	GpStatus = gp_init( MyGpibAdrs, 0, 0 );
#endif
	if( GpStatus != 0 )
	{
		printf("gp_init() error [%d]\n", GpStatus);
		return;
	}

	// enable IFC
#ifdef _USE_U2
	GpStatus = U2Gpcli(fd);
#else
	GpStatus = gp_cli();
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_cli() error [%d]\n", GpStatus);
		return;
	}

	// enable REN
#ifdef _USE_U2
	GpStatus = U2Gpren(fd);
#else
	GpStatus = gp_ren();
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_ren() error [%d]\n", GpStatus);
		return;
	}

	// Send SDC command
	// get address of GPIBdevice
#ifdef _USE_U2
	GpStatus = U2Gpclr(fd, GpAdrsBuf);
#else
	GpStatus = gp_clr( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_clr() error [%d]\n", GpStatus);
		return;
	}

	// set 15 second GPIB bus timeout
#ifdef _USE_U2
	GpStatus = U2Gptmout(fd, 15);
#else
	GpStatus = gp_tmout(15);
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_tmout() error [%d]\n", GpStatus);
		return;
	}

	GpStatus = SC7201Command();
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return;
	}

	InitFlag = 1;
	if(timer){
		;
	}else{
		timer = gtk_timeout_add (2000, timeout_callback, data);
	}
}

static void Cmd_OnCancel( GtkWidget *widget,
                      gpointer   data )
{
	if(timer){
		gtk_timeout_remove (timer);
		timer = 0;
	}
}

static void enter_adrs( GtkWidget *widget,
                            GtkWidget *entry )
{
  const gchar *entry_text;

  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  //printf ("changed contents: %s\n", entry_text);
  memset(GpAdrsBuf, 0x00, sizeof(GpAdrsBuf));
  sprintf(GpAdrsBuf, "%s", entry_text);
}


static void Cmd_OnCmdCheck( GtkWidget    *widget,
                      gpointer   data )
{
	int		GpStatus;

	if(InitFlag == 0){
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
		return;
	}
	if(timer){
		gtk_timeout_remove (timer);
		timer = 0;
	}

	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
	{
		/* もしプログラムの流れがここに到達した場合、トグル・ボタンは下がった状態です。*/
#ifdef _USE_U2
		GpStatus = U2Gpwrt(fd, GpAdrsBuf, "F0");
#else
		GpStatus = gp_wrt( GpAdrsBuf, "F0" );
#endif
		if ( GpStatus != 0 )
		{
			printf("gp_wrt() error [%d]\n", GpStatus);
			return;
		}
	} else {
		/* もしプログラムの流れがここに到達した場合、トグル・ボタンは上がった状態です。*/
#ifdef _USE_U2
		GpStatus = U2Gpclr(fd, GpAdrsBuf);
#else
		GpStatus = gp_clr( GpAdrsBuf );
#endif
		if ( GpStatus != 0 )
		{
			printf("gp_clr() error [%d]\n", GpStatus);
			return;
		}
	}

	GpStatus = SC7201Command();
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return;
	}

	if( RangeFlag != 6 && RangeFlag != 7 && RangeFlag != 8 )
		timer = gtk_timeout_add (2000, timeout_callback, data);
}

static void Cmd_OnCmdRange( GtkWidget *widget,
                      gpointer   data )
{
	int		GpStatus;

	if(InitFlag == 0){
		gtk_combo_box_set_active (GTK_COMBO_BOX (widget), 1);
		return;
	}

	if( RangeFlag != gtk_combo_box_get_active (GTK_COMBO_BOX (widget)) )
	{
		if(timer){
			gtk_timeout_remove (timer);
			timer = 0;
		}
		if( ( RangeFlag == 7 || RangeFlag == 8 ) && gtk_combo_box_get_active (GTK_COMBO_BOX (widget)) == 6 )
			return;
		RangeFlag = gtk_combo_box_get_active (GTK_COMBO_BOX (widget));
		if( RangeFlag == 6 )
		{
			//EnableWindow( GetDlgItem(hwnd, IDB_OPEN), TRUE );
			memset(szBuf, 0x00, sizeof(szBuf));
			gtk_statusbar_push (GTK_STATUSBAR (counter_bar), GPOINTER_TO_INT (counter_id), szBuf);
		}
		GpStatus = SC7201Command();
		if ( GpStatus != 0 )
		{
			printf("gp_wrt() error [%d]\n", GpStatus);
			return;
		}
		if( RangeFlag != 6 )
		{
			//EnableWindow( GetDlgItem(hwnd, IDB_OPEN), FALSE );
			timer = gtk_timeout_add (2000, timeout_callback, data);
		}
	}
}

static void Cmd_OnCmdOpen( GtkWidget    *widget,
                      gpointer   data )
{
	int		GpStatus;

	if(InitFlag == 0){
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
		return;
	}

	if( RangeFlag != 6 && RangeFlag != 7 && RangeFlag != 8 ){
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
		return;
	}

	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
    {
		/* もしプログラムの流れがここに到達した場合、トグル・ボタンは下がった状態です。*/
		RangeFlag = 7;
	} else {
		/* もしプログラムの流れがここに到達した場合、トグル・ボタンは上がった状態です。*/
		RangeFlag = 8;
	}

	GpStatus = SC7201Command();
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return;
	}
}

static void Cmd_OnCmdTime( GtkWidget *widget,
                      gpointer   data )
{
	int		GpStatus;

	if(InitFlag == 0){
		gtk_combo_box_set_active (GTK_COMBO_BOX (widget), 1);
		return;
	}

	if( TimeFlag != gtk_combo_box_get_active (GTK_COMBO_BOX (widget)) )
	{
		if(timer){
			gtk_timeout_remove (timer);
			timer = 0;
		}
		TimeFlag = gtk_combo_box_get_active (GTK_COMBO_BOX (widget));
		GpStatus = SC7201Command();
		if ( GpStatus != 0 )
		{
			printf("gp_wrt() error [%d]\n", GpStatus);
			return;
		}
		if( RangeFlag != 6 && RangeFlag != 7 && RangeFlag != 8 )
			timer = gtk_timeout_add (2000, timeout_callback, data);
	}
}

static void Cmd_OnCmdDiff( GtkWidget    *widget,
                      gpointer   data )
{
	int		GpStatus;

	if(InitFlag == 0){
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), FALSE);
		return;
	}
	if(timer){
		gtk_timeout_remove (timer);
		timer = 0;
	}

	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget))) 
	{
		/* もしプログラムの流れがここに到達した場合、トグル・ボタンは下がった状態です。*/
		DiffFlag = 1;
	} else {
		/* もしプログラムの流れがここに到達した場合、トグル・ボタンは上がった状態です。*/
		DiffFlag = 0;
	}

	GpStatus = SC7201Command();
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		return;
	}

	if( RangeFlag != 6 && RangeFlag != 7 && RangeFlag != 8 )
		timer = gtk_timeout_add (2000, timeout_callback, data);
}

static void Cmd_OnCmdReset( GtkWidget *widget,
                      gpointer   data )
{
	int		GpStatus;

	if(InitFlag == 0){
		return;
	}
	if(timer){
		gtk_timeout_remove (timer);
		timer = 0;
	}

	if( DiffFlag == 0 )
	{
#ifdef _USE_U2
		GpStatus = U2Gplcl(fd, GpAdrsBuf);
#else
		GpStatus = gp_lcl( GpAdrsBuf );
#endif
		if ( GpStatus != 0 )
		{
			printf("gp_lcl() error [%d]\n", GpStatus);
			return;
		}
	}
	if( DiffFlag == 1 )
	{
#ifdef _USE_U2
		GpStatus = U2Gpwrt(fd, GpAdrsBuf, "G1");
#else
		GpStatus = gp_wrt( GpAdrsBuf, "G1" );
#endif
		if ( GpStatus != 0 )
		{
			printf("gp_wrt() error [%d]\n", GpStatus);
			return;
		}
		GpStatus = SC7201Command();
		if ( GpStatus != 0 )
		{
			printf("gp_wrt() error [%d]\n", GpStatus);
			return;
		}

		if( RangeFlag != 6 && RangeFlag != 7 && RangeFlag != 8 )
			timer = gtk_timeout_add (2000, timeout_callback, data);
	}
}

static void create_window( void )
{
	GtkWidget *window;
	GtkWidget *hbox;
	GtkWidget *vbox;
	GtkWidget *frame;
	GtkWidget *label;
	GtkWidget *entry;
	GtkWidget *status_bar;
	GtkWidget *combo_box;
	GtkWidget *button;
	gint context_id;
	char	buf[12];

    /* make new window */
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_container_set_border_width (GTK_CONTAINER (window), 5);
    gtk_widget_set_size_request (GTK_WIDGET (window), 360, 240);
    gtk_window_set_title (GTK_WINDOW (window), "sc7201");
    g_signal_connect (G_OBJECT (window), "destroy",
		      G_CALLBACK (gtk_main_quit), NULL);

    vbox = gtk_vbox_new (FALSE, 2);
    gtk_container_add (GTK_CONTAINER (window), vbox);
    gtk_widget_show (vbox);

    /* make GPIB adrs */
    hbox = gtk_hbox_new (FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (hbox), 2);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
    gtk_widget_show (hbox);

    label = gtk_label_new ("GPIB 機器アドレス");
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 5);
    gtk_widget_show (label);

    entry = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (entry), 1);
    g_signal_connect (G_OBJECT (entry), "changed",
		      G_CALLBACK (enter_adrs),
		      (gpointer) entry);
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, "4");
    gtk_entry_set_text (GTK_ENTRY (entry), buf);
    gtk_box_pack_start (GTK_BOX (hbox), entry, FALSE, FALSE, 0);
    gtk_widget_show (entry);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (hbox), 2);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
    gtk_widget_show (hbox);

    label = gtk_label_new ("REX-5052 アドレス");
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 5);
    gtk_widget_show (label);

    status_bar = gtk_statusbar_new ();
    context_id = gtk_statusbar_get_context_id(
                          GTK_STATUSBAR (status_bar), "Statusbar example");
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, " %d", MyGpibAdrs);
    gtk_statusbar_push (GTK_STATUSBAR (status_bar), context_id, buf);
    gtk_box_pack_start (GTK_BOX (hbox), status_bar, TRUE, TRUE, 0);
    gtk_widget_show (status_bar);

    /* 新しいフレームを作成します。*/
    frame = gtk_frame_new ("SC7201 UNIVERSAL COUNTER");
    gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
    gtk_widget_show (frame);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
    gtk_container_add (GTK_CONTAINER (frame), hbox);
    gtk_widget_show (hbox);

    label = gtk_label_new ("計測値");
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 5);
    gtk_widget_show (label);

    counter_bar = gtk_statusbar_new ();
    counter_id = gtk_statusbar_get_context_id(
                          GTK_STATUSBAR (counter_bar), "Statusbar example");
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, " ", MyGpibAdrs);
    gtk_statusbar_push (GTK_STATUSBAR (counter_bar), counter_id, buf);
    gtk_box_pack_start (GTK_BOX (hbox), counter_bar, TRUE, TRUE, 0);
    gtk_widget_show (counter_bar);

    /* make botton */
    hbox = gtk_hbox_new (FALSE, 2);
    gtk_container_set_border_width (GTK_CONTAINER (hbox), 2);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
    gtk_widget_show (hbox);

	button = gtk_toggle_button_new_with_label ("CHECK");
	g_signal_connect (G_OBJECT (button), "toggled",
                      G_CALLBACK (Cmd_OnCmdCheck), NULL);
	gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
	gtk_widget_show (button);              

    combo_box = gtk_combo_box_new_text();
    gtk_combo_box_append_text(GTK_COMBO_BOX (combo_box), "FREQ A 200MHz");
    gtk_combo_box_append_text(GTK_COMBO_BOX (combo_box), "FREQ A 16MHz");
    gtk_combo_box_append_text(GTK_COMBO_BOX (combo_box), "PERIOD A");
    gtk_combo_box_append_text(GTK_COMBO_BOX (combo_box), "f1/f2 A/B");
    gtk_combo_box_append_text(GTK_COMBO_BOX (combo_box), "TIME INT A->B");
    gtk_combo_box_append_text(GTK_COMBO_BOX (combo_box), "DUTY A");
    gtk_combo_box_append_text(GTK_COMBO_BOX (combo_box), "MANUAL A");
    gtk_combo_box_set_active (GTK_COMBO_BOX (combo_box), 1);
    g_signal_connect (G_OBJECT (combo_box), "changed",
                      G_CALLBACK (Cmd_OnCmdRange), NULL);
    gtk_box_pack_start (GTK_BOX (hbox), combo_box, FALSE, FALSE, 0);
    gtk_widget_show (combo_box);

	button = gtk_toggle_button_new_with_label ("OPEN");
	g_signal_connect (G_OBJECT (button), "toggled",
                      G_CALLBACK (Cmd_OnCmdOpen), NULL);
	gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
	gtk_widget_show (button);              

    /* make botton */
    hbox = gtk_hbox_new (FALSE, 2);
    gtk_container_set_border_width (GTK_CONTAINER (hbox), 2);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
    gtk_widget_show (hbox);

    combo_box = gtk_combo_box_new_text();
    gtk_combo_box_append_text(GTK_COMBO_BOX (combo_box), "1ms");
    gtk_combo_box_append_text(GTK_COMBO_BOX (combo_box), "0.1s");
    gtk_combo_box_append_text(GTK_COMBO_BOX (combo_box), "1s");
    gtk_combo_box_append_text(GTK_COMBO_BOX (combo_box), "10s");
    gtk_combo_box_append_text(GTK_COMBO_BOX (combo_box), "EXT GATEB");
    gtk_combo_box_set_active (GTK_COMBO_BOX (combo_box), 1);
    g_signal_connect (G_OBJECT (combo_box), "changed",
                      G_CALLBACK (Cmd_OnCmdTime), NULL);
    gtk_box_pack_start (GTK_BOX (hbox), combo_box, FALSE, FALSE, 0);
    gtk_widget_show (combo_box);

	button = gtk_toggle_button_new_with_label ("DIFF");
	g_signal_connect (G_OBJECT (button), "toggled",
                      G_CALLBACK (Cmd_OnCmdDiff), NULL);
	gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
	gtk_widget_show (button);              

	button = gtk_button_new_with_label ("RESET");
	g_signal_connect (G_OBJECT (button), "clicked",
                      G_CALLBACK (Cmd_OnCmdReset), NULL);
	gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
	gtk_widget_show (button);              

    /* make botton */
    hbox = gtk_hbox_new (FALSE, 2);
    gtk_container_set_border_width (GTK_CONTAINER (hbox), 2);
    gtk_box_pack_end (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
    gtk_widget_show (hbox);

	button = gtk_button_new_with_label ("初期化");
	g_signal_connect (G_OBJECT (button), "clicked",
                      G_CALLBACK (Cmd_OnCmdGpInit), NULL);
	gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
	gtk_widget_show (button);              

	button = gtk_button_new_with_label ("終了");
	g_signal_connect (G_OBJECT (button), "clicked",
		               G_CALLBACK (Cmd_OnCancel), NULL);
	g_signal_connect (G_OBJECT (button), "clicked",
			      G_CALLBACK (gtk_main_quit), NULL);
	gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
	gtk_widget_grab_default (button);
	gtk_widget_show (button);

	gtk_widget_show (window);
}

int main( int   argc,
          char *argv[] )
{
#ifdef _USE_U2
	int id, i;
	unsigned short UnitId[MAX_DEVICES];
	unsigned short MaxUnit = MAX_DEVICES;

	// 引数無しの場合
	if (argc != 2) {
		MaxUnit = U2GpEnumUnit(UnitId, MaxUnit);
		for(i=0;i<MaxUnit;i++){
			printf("U2GpEnumUnit%d UnitId = %02x \n", i, UnitId[i]);
		}
		//最初に見つけたREX-5052をオープン
		fd=U2GpOpenUnit(UnitId[0]);
	}else{
		id = atoi (argv[1]);
		//入力されたIDをもつREX-5052をオープン
		fd=U2GpOpenUnit(id);
	}
#else
	fd=gp_attach();
#endif
	if(fd<=0){
		printf("Device not opened \n");
		return 0;
    }
	InitFlag = 0;
	RangeFlag = 1;
	TimeFlag = 1;
	DiffFlag = 0;
	//gp_GetDeviceID(fd, 0, (char*)&MyGpibAdrs);

	gtk_init (&argc, &argv);

	create_window();

	gtk_main ();
	gp_detach(fd);

	return 0;
}
