/**********************************************************************************
 *
 *	電圧発生器7651 制御プログラム
 *
 *	機能 ： 横河標準電圧発生器7651を制御して、正弦波を作成する
 *
 **********************************************************************************/
//gcc -g yg7651.c -o yg7651 -L/usr/local/lib/ -lrex5052_gpib -lpthread \
//	`pkg-config gtk+-2.0 --cflags --libs`
// caution: You need ROOT previledge when execute this utility

#include <stdlib.h>
#include <math.h>
//#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <glib.h>

#include "rex5052_gpib.h"

//#define _USE_U2
#define MAX_DEVICES				16


int fd=0;
short MyGpibAdrs = 1;// GPIB address for PCMCIA-GPIB
char GpAdrsBuf[12];	// GPIB address for target-GPIB
int InitFlag;

short GpMachineAdrs;	// 接続するGPIB機器アドレス
static int Count;		// データカウンタ
char RunFlag;			// タイマーフラグ
char RecvBuf[32];		// 7651からデータ受信用バッファ
double PlotVal;		// 現在横河7651の出力電圧値

double SampFreq; // サンプリング周期
double SinFreq; // 正弦波周期
double SinMag; // 正弦波振幅

int timer = 0;
GtkWidget *status_bar;
gint status_id;
//struct GdkGC *gdkgc;

int g_Starttim2;
int g_Stoptim2;
unsigned char g_ovFlg2 = 0;

int GetTime2()
{
	struct timeval tv;
	struct timezone tz;
	int		Nowtim;

	gettimeofday(&tv, &tz);
	Nowtim = ((tv.tv_sec % 60) * 1000 ) + (tv.tv_usec / 1000 );
	return Nowtim;
}

void SetTimeOut2(int msec)
{
	int		cnt;
	g_Starttim2 = GetTime2();

	g_ovFlg2 = 0;
	g_Stoptim2 = g_Starttim2;
	for(cnt = 1; cnt <= msec; cnt++)
	{
		g_Stoptim2++; // +1
		if(g_Stoptim2 == 60001)
			g_ovFlg2 = 1;
	}
	//printf("g_Starttim2 = %d, g_Stoptim2 = %d\n", g_Starttim2, g_Stoptim2);
}

unsigned char IsTimeOut2(void)
{
	int		Nowtim;

	Nowtim = GetTime2();

	if(g_ovFlg2 == 1){
		if(Nowtim < 30000){
			Nowtim = Nowtim + 60000;
		}
	}

	if(g_Stoptim2 <= Nowtim)
		return 1;
	else
		return 0;
}

static int InitGpMachine( void )
{
	int		GpStatus;

	// Init GPIB controller
#ifdef _USE_U2
	GpStatus = U2Gpinit( fd, MyGpibAdrs );
#else
	GpStatus = gp_init( MyGpibAdrs, 0, 0 );
#endif
	if( GpStatus != 0 )
	{
		printf("gp_init() error [%d]\n", GpStatus);
		return -1;
	}

	// enable IFC
#ifdef _USE_U2
	GpStatus = U2Gpcli(fd);
#else
	GpStatus = gp_cli();
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_cli() error [%d]\n", GpStatus);
		return -1;
	}

	// enable REN
#ifdef _USE_U2
	GpStatus = U2Gpren(fd);
#else
	GpStatus = gp_ren();
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_ren() error [%d]\n", GpStatus);
		return -1;
	}

	// Send SDC command
	// get address of GPIBdevice
#ifdef _USE_U2
	GpStatus = U2Gpclr(fd, GpAdrsBuf);
#else
	GpStatus = gp_clr( GpAdrsBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_clr() error [%d]\n", GpStatus);
		return -1;
	}

	gp_wait( 1 );
	//gp_setdelay( 1 );
	//gp_delay( 1 );

	InitFlag = 1;
	return 0;
}

static int SetProgFor7651( void )
{
	int		GpStatus;
	char	ComdBuf[32];
	char	IntvBuf[9];
	char	InBuf[32];
	double	SampLen;
	double	Val;
	double	Loop;

	SampLen   = SinFreq / SampFreq;

	sprintf( IntvBuf, "PI%06.1lf", SampFreq );
	IntvBuf[8] = '\0';	//???

#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, "PRS");
#else
	GpStatus = gp_wrt( GpAdrsBuf, "PRS" );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		goto GPIB_ERROR;
	}

	for(Loop=0.0; Loop<SampLen; Loop++)
	{
		// 
		Val = SinMag * sin( 2.0*3.1415926535 * ( 1.0/SinFreq ) * (Loop*SampFreq) );
		if( Val == 10.0 )
			sprintf( InBuf, "%+6.4lf", Val );
		else 
			sprintf( InBuf, "%+6.5lf", Val );
		memset( ComdBuf, 0x00, sizeof( ComdBuf ) );
		strcpy( ComdBuf, "F1R5S");
		strcat( ComdBuf, InBuf );
		strcat( ComdBuf, "E+0" );
#ifdef _USE_U2
		GpStatus = U2Gpwrt(fd, GpAdrsBuf, ComdBuf);
#else
		GpStatus = gp_wrt( GpAdrsBuf, ComdBuf );
#endif
		if ( GpStatus != 0 )
		{
			printf("gp_wrt() error [%d]\n", GpStatus);
			goto GPIB_ERROR;
		}
	}

#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, "PRE");
#else
	GpStatus = gp_wrt( GpAdrsBuf, "PRE" );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		goto GPIB_ERROR;
	}

#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, IntvBuf);
#else
	GpStatus = gp_wrt( GpAdrsBuf, IntvBuf );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		goto GPIB_ERROR;
	}

#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, "SW0");
#else
	GpStatus = gp_wrt( GpAdrsBuf, "SW0" );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		goto GPIB_ERROR;
	}

#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, "M0");
#else
	GpStatus = gp_wrt( GpAdrsBuf, "M0" );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		goto GPIB_ERROR;
	}

#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, "F1R5S+0E");
#else
	GpStatus = gp_wrt( GpAdrsBuf, "F1R5S+0E" );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		goto GPIB_ERROR;
	}

#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, "O1E");
#else
	GpStatus = gp_wrt( GpAdrsBuf, "O1E" );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		goto GPIB_ERROR;
	}

#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, "MS19");
#else
	GpStatus = gp_wrt( GpAdrsBuf, "MS19" );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		goto GPIB_ERROR;
	}

	return 0;

GPIB_ERROR:
	return -1;
}

static int GetDataFrom7651( void )
{
	int		GpStatus;
	char	DataBuf[32];
	unsigned char	StatusByte;
	int				i, j;
	char szBuf[256];

#ifdef _USE_U2
	GpStatus = U2Gpwrt(fd, GpAdrsBuf, "OD");
#else
	GpStatus = gp_wrt( GpAdrsBuf, "OD" );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_wrt() error [%d]\n", GpStatus);
		goto STOP;
	}

	//SetTimeOut2(SampFreq*900);
	StatusByte = 0;
	while( (StatusByte != 0x41) && (StatusByte != 0x51) )
	{
#ifdef _USE_U2
		if( U2Gprds( fd, GpAdrsBuf, &StatusByte ) != 0)
#else
		if( gp_rds( GpAdrsBuf, &StatusByte ) != 0)
#endif
		{
			printf("gp_rds() error\n");
			goto STOP;
		}
		//if(IsTimeOut2()){
		//	printf("gp_rds() timeout\n");
		//	return 0;
		//}
	}

#ifdef _USE_U2
	GpStatus = U2Gpred(fd, GpAdrsBuf, RecvBuf, sizeof(RecvBuf));
#else
	GpStatus = gp_red( GpAdrsBuf, RecvBuf, sizeof(RecvBuf) );
#endif
	if ( GpStatus != 0 )
	{
		printf("gp_red() error [%d]\n", GpStatus);
		goto STOP;
	}

	Count += 1;
	for(i=4, j=0; RecvBuf[i] != ','; i++, j++  ){
		if(i< 32){
			DataBuf[j] = RecvBuf[i];
		}
	}

	if(i< 32){
		PlotVal = atof( DataBuf );
	}
	printf("PlotVal [%+6.4lf], Count [%d] \n", PlotVal, Count);
	sprintf( szBuf, "現在電圧値： %+6.4lf Volt     カウンタ： %lu", PlotVal, Count );
	gtk_statusbar_push (GTK_STATUSBAR (status_bar), GPOINTER_TO_INT (status_id), szBuf);

	return 0;

STOP:
	return -1;
}

static int CheckParam( void )
{
	if( (MyGpibAdrs > 30) || (GpMachineAdrs > 30) )
	{
		printf("GPIBアドレスは30より小さく設定してください。\n");
		return -1;
	}
	if( GpMachineAdrs == MyGpibAdrs )
	{
		printf("GPIBアドレスは重複しないように設定してください。\n");
		return -1;
	}
	if( ( SinFreq >= 3600.0 ) || ( SinFreq < 1.0 ) )
	{
		printf("正弦波周期は(1.0, 3600.0)以内に設定してください。\n");
		return -1;
	}
	// サンプリング周期は横河7651のプログラム実行時のインターバル
	if( SampFreq < 0.5 )
	{
		printf("サンプリング周期は0.5秒以上に設定してください。\n");
		return -1;
	}
	// 1周期内のサンプリング点数＝横河7651のプログラムカウンタ数。50以下にする
	if( ( SampFreq > (0.5 * SinFreq) ) || ( SinFreq > (50.0 * SampFreq) ) )
	{
		printf("サンプリング周期は正弦波周期の(1/50,1/2)以内に設定してください。\n");
		return -1;
	}
	if( ( SinMag > 10.0 ) || ( SinMag < 0.0 ) )
	{
		printf("正弦波の振幅は(0,10)以内に設定してください。\n");
		return -1;
	}

	return 0;
}

static gboolean timeout_callback( gpointer data )
{
	//printf("GetTime2() = %d\n", GetTime2());

	if( GetDataFrom7651() != 0 )
	{
		//gtk_timeout_remove (timer);
		timer = 0;
		RunFlag = 0;
		return FALSE;
	}

	//PlotDataWave( hwnd, PlotVal );

	/* これがタイムアウト関数なので、
	* この関数が連続して呼び出されるように TRUE を返します。*/
	return TRUE;
}

static void Cmd_OnCmdGpInit( GtkWidget *widget,
                       gpointer   data )
{
	Count = 0;

	if( RunFlag == 1 )
	{
		if(timer){
			gtk_timeout_remove (timer);
			timer = 0;
		}
#ifdef _USE_U2
		U2Gpwrt(fd, GpAdrsBuf, "RU0");
#else
		gp_wrt( GpAdrsBuf, "RU0" );
#endif
		RunFlag = 0;
	}

	if( CheckParam() != 0 )
	{
		return;
	}

	if( InitGpMachine() != 0 )
	{
		return;
	}

	if( SetProgFor7651() != 0 )
	{
		return;
	}
}

static void Cmd_OnCmdOutput( GtkWidget *widget,
                       gpointer   data )
{
	unsigned char	StatusByte;
	char	DataBuf[32];

	if(InitFlag == 0){
		return;
	}
	if( RunFlag == 1 )
	{
		return;
	}

	//if(InPort(0x2) & 0x02){
	//	gp_rds( GpAdrsBuf, &StatusByte );
	//	printf("StatusByte [%x]\n", StatusByte);
	//	gp_red( GpAdrsBuf, DataBuf, sizeof(DataBuf) );
	//}
	//printf("ISR1 [%x]\n", InPort(0x2));

#ifdef _USE_U2
	if( U2Gpwrt(fd, GpAdrsBuf, "RU2" ) != 0 )
#else
	if( gp_wrt( GpAdrsBuf, "RU2" ) != 0 )
#endif
	{
		printf("gp_wrt() error \n");
		return;
	}

	if( GetDataFrom7651() != 0 )
	{
		printf("Cmd_OnCmdOutput:GetDataFrom7651() error \n");
		return;
	}

	timer = gtk_timeout_add (SampFreq*1000, timeout_callback, data);

	RunFlag = 1;
}

static void Cmd_OnCancel( GtkWidget *widget,
                      gpointer   data )
{
	printf("Cmd_OnCancel \n");

	if( RunFlag == 1 ){
		if(timer){
			gtk_timeout_remove (timer);
			timer = 0;
		}
#ifdef _USE_U2
		U2Gpwrt(fd, GpAdrsBuf, "RU0");
#else
		gp_wrt( GpAdrsBuf, "RU0" );
#endif
		RunFlag = 0;
	}

	if(InitFlag){
		//send LCL command
#ifdef _USE_U2
		U2Gplcl(fd, "");
#else
		gp_lcl( "" );
#endif
	}
}

static void enter_adrs( GtkWidget *widget,
                            GtkWidget *entry )
{
  const gchar *entry_text;

  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  memset(GpAdrsBuf, 0x00, sizeof(GpAdrsBuf));
  sprintf(GpAdrsBuf, "%s", entry_text);
  GpMachineAdrs = atoi(GpAdrsBuf);
  //printf ("GpMachineAdrs: %d\n", GpMachineAdrs);
}

static void enter_sampfreq( GtkWidget *widget,
                            GtkWidget *entry )
{
  const gchar *entry_text;
  char	buf[12];

  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  sprintf(buf, "%s", entry_text);
  SampFreq = atof(buf);
  //printf ("SampFreq: %3.1f\n", SampFreq);
}

static void enter_sinfreq( GtkWidget *widget,
                            GtkWidget *entry )
{
  const gchar *entry_text;
  char	buf[12];

  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  sprintf(buf, "%s", entry_text);
  SinFreq = atof(buf);
  //printf ("SinFreq: %3.1f\n", SinFreq);
}

static void enter_sinmag( GtkWidget *widget,
                            GtkWidget *entry )
{
  const gchar *entry_text;
  char	buf[12];

  entry_text = gtk_entry_get_text (GTK_ENTRY (entry));
  sprintf(buf, "%s", entry_text);
  SinMag = atof(buf);
  //printf ("SinMag: %3.1f\n", SinMag);
}

static void create_window( void )
{
	GtkWidget *window;
	GtkWidget *table;
	GtkWidget *table1;
	GtkWidget *frame;
	GtkWidget *hbox;
	GtkWidget *label;
	GtkWidget *entry;
	GtkWidget *temp_bar;
	GtkWidget *area;
	GtkWidget *vrule;
	GtkWidget *button;
	gint temp_id;
	char	buf[12];

    /* make new window */
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_container_set_border_width (GTK_CONTAINER (window), 5);
    gtk_widget_set_size_request (GTK_WIDGET (window), 720, 360);
    gtk_window_set_title (GTK_WINDOW (window), "横河7651");
    g_signal_connect (G_OBJECT (window), "destroy",
		      G_CALLBACK (gtk_main_quit), NULL);

    /* 縦横 12x12 のテーブルを生成します。*/
    table = gtk_table_new (12, 12, TRUE);
    gtk_container_add (GTK_CONTAINER (window), table);
    gtk_widget_show (table);

    /* make GPIB adrs */
    /* 新しいフレームを作成します。*/
    frame = gtk_frame_new ("GPIB 機器アドレス設定");
    gtk_table_attach_defaults (GTK_TABLE (table), frame, 0, 4, 0, 3);
    gtk_widget_show (frame);

    /* 縦横 4x4 のテーブルを生成します。*/
    table1 = gtk_table_new (4, 4, TRUE);
    gtk_container_set_border_width (GTK_CONTAINER (table1), 5);
    gtk_table_set_row_spacing( GTK_TABLE (table1), 0, 5 );
    gtk_container_add (GTK_CONTAINER (frame), table1);
    gtk_widget_show (table1);

    label = gtk_label_new ("横河7651 アドレス");
    gtk_table_attach_defaults (GTK_TABLE (table1), label, 0, 3, 0, 2);
    gtk_widget_show (label);

    entry = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (entry), 2);
    g_signal_connect (G_OBJECT (entry), "changed",
		      G_CALLBACK (enter_adrs),
		      (gpointer) entry);
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, "5");
    gtk_entry_set_text (GTK_ENTRY (entry), buf);
    gtk_table_attach_defaults (GTK_TABLE (table1), entry, 3, 4, 0, 2);
    gtk_widget_show (entry);

    label = gtk_label_new ("REX-5052 アドレス");
    gtk_table_attach_defaults (GTK_TABLE (table1), label, 0, 3, 2, 4);
    gtk_widget_show (label);

    temp_bar = gtk_statusbar_new ();
    temp_id = gtk_statusbar_get_context_id(
                          GTK_STATUSBAR (temp_bar), "Statusbar example");
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, " %d", MyGpibAdrs);
    gtk_statusbar_push (GTK_STATUSBAR (temp_bar), temp_id, buf);
    gtk_table_attach_defaults (GTK_TABLE (table1), temp_bar, 3, 4, 2, 4);
    gtk_widget_show (temp_bar);

    /* 新しいフレームを作成します。*/
    frame = gtk_frame_new ("出力正弦波設定");
    gtk_table_attach_defaults (GTK_TABLE (table), frame, 0, 4, 6, 11);
    gtk_widget_show (frame);

    /* 縦横 4x4 のテーブルを生成します。*/
    table1 = gtk_table_new (4, 4, TRUE);
    gtk_container_set_border_width (GTK_CONTAINER (table1), 5);
    gtk_table_set_col_spacing( GTK_TABLE (table1), 1, 2 );
    gtk_container_add (GTK_CONTAINER (frame), table1);
    gtk_widget_show (table1);

    label = gtk_label_new ("出力電圧レンジ");
    gtk_table_attach_defaults (GTK_TABLE (table1), label, 0, 2, 0, 1);
    gtk_widget_show (label);

    temp_bar = gtk_statusbar_new ();
    temp_id = gtk_statusbar_get_context_id(
                          GTK_STATUSBAR (temp_bar), "Statusbar example");
    gtk_statusbar_push (GTK_STATUSBAR (temp_bar), temp_id, "-10V 〜 +10V");
    gtk_table_attach_defaults (GTK_TABLE (table1), temp_bar, 2, 4, 0, 1);
    gtk_widget_show (temp_bar);

    label = gtk_label_new ("サンプリング周期");
    gtk_table_attach_defaults (GTK_TABLE (table1), label, 0, 2, 1, 2);
    gtk_widget_show (label);

    entry = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (entry), 6);
    g_signal_connect (G_OBJECT (entry), "changed",
		      G_CALLBACK (enter_sampfreq),
		      (gpointer) entry);
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, "0.5");
    gtk_entry_set_text (GTK_ENTRY (entry), buf);
    gtk_table_attach_defaults (GTK_TABLE (table1), entry, 2, 3, 1, 2);
    gtk_widget_show (entry);

    label = gtk_label_new ("Sec");
    gtk_table_attach_defaults (GTK_TABLE (table1), label, 3, 4, 1, 2);
    gtk_widget_show (label);

    label = gtk_label_new ("正弦波周期");
    gtk_table_attach_defaults (GTK_TABLE (table1), label, 0, 2, 2, 3);
    gtk_widget_show (label);

    entry = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (entry), 6);
    g_signal_connect (G_OBJECT (entry), "changed",
		      G_CALLBACK (enter_sinfreq),
		      (gpointer) entry);
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, "25");
    gtk_entry_set_text (GTK_ENTRY (entry), buf);
    gtk_table_attach_defaults (GTK_TABLE (table1), entry, 2, 3, 2, 3);
    gtk_widget_show (entry);

    label = gtk_label_new ("Sec");
    gtk_table_attach_defaults (GTK_TABLE (table1), label, 3, 4, 2, 3);
    gtk_widget_show (label);

    label = gtk_label_new ("正弦波振幅");
    gtk_table_attach_defaults (GTK_TABLE (table1), label, 0, 2, 3, 4);
    gtk_widget_show (label);

    entry = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (entry), 4);
    g_signal_connect (G_OBJECT (entry), "changed",
		      G_CALLBACK (enter_sinmag),
		      (gpointer) entry);
    memset(buf, 0x00, sizeof(buf));
    sprintf(buf, "10");
    gtk_entry_set_text (GTK_ENTRY (entry), buf);
    gtk_table_attach_defaults (GTK_TABLE (table1), entry, 2, 3, 3, 4);
    gtk_widget_show (entry);

    label = gtk_label_new ("Sec");
    gtk_table_attach_defaults (GTK_TABLE (table1), label, 3, 4, 3, 4);
    gtk_widget_show (label);

    /* make graph */
    label = gtk_label_new ("[Volt]");
    gtk_table_attach_defaults (GTK_TABLE (table), label, 4, 6, 0, 1);
    gtk_widget_show (label);

    label = gtk_label_new ("計測データ波形");
    gtk_table_attach_defaults (GTK_TABLE (table), label, 6, 10, 0, 1);
    gtk_widget_show (label);

    /* ルーラを配置するテーブルと、ドローイング・エリアを作成します。*/
    table1 = gtk_table_new (3, 2, FALSE);
    gtk_table_attach_defaults (GTK_TABLE (table), table1, 5, 12, 1, 10);
    gtk_container_set_border_width (GTK_CONTAINER (table1), 5);
    //gtk_table_set_col_spacing( GTK_TABLE (table1), 1, 2 );
    gtk_widget_show (table1);

    area = gtk_drawing_area_new ();
    //gtk_widget_set_size_request (GTK_WIDGET (area), 300, 200);
    gtk_drawing_area_size (GTK_DRAWING_AREA (area), 300, 200);
    gtk_table_attach (GTK_TABLE (table1), area, 1, 2, 1, 2,
                      GTK_EXPAND|GTK_FILL, GTK_FILL, 0, 0);
    //gtk_widget_set_events (area, GDK_POINTER_MOTION_MASK |
    //                             GDK_POINTER_MOTION_HINT_MASK);
    gtk_widget_show (area);

    //gdkgc = gdk_gc_new(GDK_WINDOW (area));

    vrule = gtk_vruler_new ();
    gtk_ruler_set_metric (GTK_RULER (vrule), GTK_PIXELS);
    gtk_ruler_set_range (GTK_RULER (vrule), 10, -10, 0, 200 );
    //g_signal_connect_swapped (G_OBJECT (area), "motion_notify_event",
    //                          G_CALLBACK (EVENT_METHOD (vrule, motion_notify_event)),
    //                          G_OBJECT (vrule));
    gtk_table_attach (GTK_TABLE (table1), vrule, 0, 1, 1, 2,
                      GTK_FILL, GTK_EXPAND|GTK_SHRINK|GTK_FILL, 0, 0);
    gtk_widget_show (vrule);

    status_bar = gtk_statusbar_new ();
    status_id = gtk_statusbar_get_context_id(
                          GTK_STATUSBAR (status_bar), "Statusbar example");
    //gtk_statusbar_push (GTK_STATUSBAR (status_bar), status_id, buf);
    gtk_table_attach_defaults (GTK_TABLE (table), status_bar, 5, 12, 10, 11);
    gtk_widget_show (status_bar);

    /* make botton */
    hbox = gtk_hbox_new (FALSE, 2);
    gtk_container_set_border_width (GTK_CONTAINER (hbox), 2);
    gtk_table_attach_defaults (GTK_TABLE (table), hbox, 5, 12, 11, 12);
    gtk_widget_show (hbox);

	button = gtk_button_new_with_label ("初期化");
	g_signal_connect (G_OBJECT (button), "clicked",
                      G_CALLBACK (Cmd_OnCmdGpInit), NULL);
	gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
	gtk_widget_show (button);              

	button = gtk_button_new_with_label ("計測開始");
	g_signal_connect (G_OBJECT (button), "clicked",
                      G_CALLBACK (Cmd_OnCmdOutput), NULL);
	gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
	gtk_widget_show (button);

	button = gtk_button_new_with_label ("終了");
	g_signal_connect (G_OBJECT (button), "clicked",
		               G_CALLBACK (Cmd_OnCancel), NULL);
	g_signal_connect (G_OBJECT (button), "clicked",
			      G_CALLBACK (gtk_main_quit), NULL);
	gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, TRUE, 0);
	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
	gtk_widget_grab_default (button);
	gtk_widget_show (button);

	gtk_widget_show (window);
}

int main( int   argc,
          char *argv[] )
{
#ifdef _USE_U2
	int id, i;
	unsigned short UnitId[MAX_DEVICES];
	unsigned short MaxUnit = MAX_DEVICES;

	// 引数無しの場合
	if (argc != 2) {
		MaxUnit = U2GpEnumUnit(UnitId, MaxUnit);
		for(i=0;i<MaxUnit;i++){
			printf("U2GpEnumUnit%d UnitId = %02x \n", i, UnitId[i]);
		}
		//最初に見つけたREX-5052をオープン
		fd=U2GpOpenUnit(UnitId[0]);
	}else{
		id = atoi (argv[1]);
		//入力されたIDをもつREX-5052をオープン
		fd=U2GpOpenUnit(id);
	}
#else
	fd=gp_attach();
#endif
	if(fd<=0){
		printf("Device not opened \n");
		return 0;
    }
	InitFlag = 0;
	//gp_GetDeviceID(fd, 0, (char*)&MyGpibAdrs);

	gtk_init (&argc, &argv);

	create_window();

	gtk_main ();
	gp_detach(fd);

	return 0;
}
